## Filename    : TestMCLAnalyser.py
## Author(s)   : Michel Le Borgne
## Created     : 03/2012
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux - IRISA/IRSET
##
"""
Unitary Tests for the MCLAnalyser
"""

import unittest
import sys, os

from cadbiom.models.clause_constraints.mcl.MCLAnalyser import MCLAnalyser
from cadbiom.models.guard_transitions.simulator.chart_simul import ChartSimulator
from cadbiom.models.guard_transitions.chart_model import ChartModel
from cadbiom.models.clause_constraints.mcl.MCLQuery import MCLSimpleQuery
from cadbiom.models.clause_constraints.mcl.MCLSolutions import FrontierSolution

TRACE_FILE = sys.stdout
#TRACE_FILE = open("/tmp/testMCLAnalyzer.txt",'w')

# helper fonctions for test automation
def lit_subsol_equal(ssol1, ssol2):
    """
    @param ssol1,ssol2: list<string>
    CAREFULL: change order in lists
    """
    if len(ssol1) != len(ssol2):
        return False
    else:
        ssol1.sort()
        ssol2.sort()
        for i in range(len(ssol1)):
            if ssol1[i] != ssol2[i]:
                return False
        return True

def lit_sol_equal(sol1, sol2):
    """
    @param sol1,sol2: list<list<string>>
    CAREFULL: change order in lists
    """
    if len(sol1) != len(sol2):
        return False
    else:
        for i in range(len(sol1)):
            if not lit_subsol_equal(sol1[i], sol2[i]):
                return False
        return True


# simple reporter
class ErrorRep(object):
    """
    error reporter of the compil type
    """
    def __init__(self):
        self.context = ""
        self.error = False
        pass

    def display(self, mess):
        """
        just printing
        """
        self.error = True
        print '\n>> '+self.context+"  "+mess

    def display_info(self, mess):
        """
        just printing
        """
        print '\n-- '+mess

    def set_context(self, cont):
        """
        for transition compiling
        """
        self.context = cont



class TestMCLAnaLyzer(unittest.TestCase):
    """
    Unit tests for MCLAnalyser
    """

    def test_lit_sol_equal(self):
        """
        Test the test helper function
        """
        sol1 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        sol2 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(res, 'Error in lit_sol equality')
        sol2 = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4'], \
                ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4'],\
                ['-_lit2', 'C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4']]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(not res, 'Error in lit_sol inequality')
        # test on numbers
        sol1 = [[], [3], [], [5]]
        sol2 = [[], [3], [], [5]]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(res, 'Error in lit_sol equality: number case')
        sol2 = [[], [3], [2], []]
        res = lit_sol_equal(sol1, sol2)
        self.assert_(not res, 'Error in lit_sol inequality: number case')

    def test_load1(self):
        """
        Load from a bcx file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_chart_file("../ucl/examples/test_tgfb_ref_300511.bcx")
        res = rep.error
        self.assert_(not res, 'Error in load1')

    def test_load2(self):
        """
        Load from a cadlang file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        file_name = "../../guard_transitions/translators/tests/tgf_cano_cmp.cal"
        mcla.build_from_cadlang(file_name)
        res = rep.error
        self.assert_(not res, 'Error in load2')

    def test_load3(self):
        """
        Load from a PID file
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_pid_file("../ucl/examples/test_plk3_pathway.xml")
        res = rep.error
        self.assert_(not res, 'Error in load3')

    def test_delay1(self):
        """
        simple delay computation
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay1.cal")

        f_prop =  "C3 and not C4"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.get_frontier())

#        if len(lsol) > 0:
#            print 'C3 ALONE'
#            print lsol[0] #first solution _ we have two
#        else:
#            print 'NO SOLUTION C4'

        res =  len(lsol) == 2
        expected_sol = [['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '_lit0', '-_lit1', 'B1'], ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4', 'B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', '-B1'], ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', '-A4', '-B2', 'B3', '_lit3', 'C2', '-C1', '-_lit0', '_lit1', '-B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C3 alone')

        f_prop = "C4 and not C3"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.get_frontier())

#        if len(lsol)>0:
#            print 'C4 ALONE', len(lsol)
#            print lsol[0]
#        else:
#            print 'NO SOLUTION C4'

        res =  len(lsol) == 1
        expected_sol = [['-_lit2', '-C4', '-h2', 'A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '-_lit0', '-_lit1', 'B1'], ['-_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', 'A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', 'B1'], ['-_lit2', '-C4', '-h2', '-A1', '-C3', 'A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', 'B1'], ['_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', '-A2', 'A4', '-B2', '-B3', '_lit3', 'C2', '-C1', '_lit0', '-_lit1', 'B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C4 alone')

        f_prop = "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)

        lsol = mcla.sq_solutions(query,
                                 4, 1, mcla.unfolder.get_frontier())

#        if len(lsol)>0:
#            print 'C3 and C4'
#            print lsol[0]
#        else:
#            print 'NO SOLUTION'

        res =  len(lsol) == 1
        expected_sol =  [['-_lit2', '-C4', '-h2', 'A1', '-C3', '-A3', '-A2', '-A4', '-B2', '-B3', '-_lit3', '-C2', 'C1', '-_lit0', '-_lit1', 'B1'], ['-_lit2', '-C4', 'h2', '-A1', '-C3', '-A3', 'A2', '-A4', '-B2', '-B3', '-_lit3', 'C2', '-C1', '_lit0', '-_lit1', 'B1'], ['-_lit2', '-C4', '-h2', '-A1', '-C3', 'A3', '-A2', '-A4', 'B2', '-B3', '-_lit3', 'C2', '-C1', '-_lit0', '-_lit1', '-B1'], ['_lit2', '-C4', '-h2', '-A1', '-C3', '-A3', '-A2', 'A4', '-B2', 'B3', '_lit3', 'C2', '-C1', '-_lit0', '_lit1', '-B1']]
        res = res and lit_sol_equal(lsol[0].__str__(), expected_sol)
        self.assert_(not res, 'Error in C3 and C4')

    def test_delay1_w_ic(self):
        """
        simple delay computation with clock sequence
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay1.cal")

        st_prop = None
        inv_prop = None
        f_prop =  "C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [], [], [3]]
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4')

        f_prop = "C4 and not C3"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [], [], [3]]
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4 and not C3')

        f_prop = "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [3], [], []]
        res =  len(lsol) != 0
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock sequence for C4')


    def test_input1(self):
        """
        simple input computation
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/input1.cal")

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and not C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        res = len(lsol) == 2
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[2], [], []]
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock/input sequence for C3 and not C4')

        f_prop = "C4 and C3"
        query = MCLSimpleQuery(None, None, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        res = len(lsol) == 1
        ic_seq = sol.extract_act_input_clock_seq()
        #print ic_seq
        expected_sol = [[], [2], [], []]
        res = res and lit_sol_equal(expected_sol, ic_seq)
        self.assert_(res, 'Error clock/input sequence for C3 and C4')


    def test_unflatten(self):
        """
        Test unflatten trajectory
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay1.cal")

        st_prop = None
        inv_prop = None
        f_prop = "C3"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        ufl = lsol[0].unflatten()
        expected_ufl = [[-1, -2, 3, -4, -5, -6, -7, -8, -9, -10, -11, -12, 13, 14, -15, 16], [-1, -2, -3, -4, -5, -6, -7, -8, 9, -10, -11, 12, -13, -14, -15, -16], [-1, -2, -3, -4, -5, -6, -7, -8, -9, 10, 11, 12, -13, -14, 15, -16]]
        res = lit_sol_equal(expected_ufl, ufl)
        self.assert_(res, 'Error unflatten')


    def test_dimacs_frontier(self):
        """
        frontier solution in DIMACS form
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        query = MCLSimpleQuery(None, None, prop)

        mcla.build_from_chart_file("../ucl/examples/tgf_211011_man.bcx")
        # lsol: list<dimacs frontier solution>
        lsol = mcla.test_dimacs_frontier_sol(query, 8, 1)
        lforbidden = []
        for sol in lsol:
            forbid = []
            for varcod in sol.frontier_values:
                if varcod > 0:
                    forbid.append(-varcod)
            lforbidden.append(forbid)
        # try another solution
        query.set_dim_start(lforbidden)
        lsol2 = mcla.test_dimacs_frontier_sol(query, 8, 1)
        # comparison: solution are ordered and must be different
        sol = lsol[0]
        sol2 = lsol2[0]
        res = True
        # remains true if all components are equal (equal solutions)
        for i in range(sol.get_solution_size()):
            res = res and (sol.get_solution()[i] == sol2.get_solution()[i])
        self.assert_(not res, 'Error dimacs_frontier: solutions not different')

    def test_prune(self):
        """
        pruning a solution
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        query = MCLSimpleQuery(None, None, prop)

        mcla.build_from_chart_file("../ucl/examples/tgf_211011_man.bcx")
        # lsol: list<dimacs frontier solution>
        lsol = mcla.test_dimacs_frontier_sol(query, 8, 10)
        small_sol = mcla.less_activated_solution(lsol)
        cpt1 = 0
        for varcod in small_sol.get_solution():
            if varcod > 0:
                cpt1 += 1
        # check solution
        start = []
        for ssol in small_sol.frontier_values:
            start.append([ssol])
        query.set_dim_start(start)
        res = mcla.sq_is_satisfiable(query, 8)
        self.assert_(res, 'Small sol is not a solution')
        # find if there are solutions with same unactivated places
        for ssol in small_sol.frontier_values:
            if ssol < 0:
                start.append([ssol])
        query.set_dim_start(start)
        res = mcla.sq_is_satisfiable(query, 8)
        self.assert_(res, 'No solution with same unactivated places')
        prune = mcla.test_prune_frontier_solution(small_sol, query, 8)
        # compare sizes
        cpt2 = 0
        for varcod in prune.get_solution():
            if varcod > 0:
                cpt2 += 1
        res = (cpt2 <= cpt1)
        self.assert_(res, 'Prune do not reduce nb of activated places')




    def test_cam_no_clock(self):
        """
        test cam without clocks
        """
        # solving
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b and p21CIP1'
        dir_n = "../../guard_transitions/translators/tests/"
        f_name = dir_n + "tgf_cano_noclock_cmp.cal"
        mcla.build_from_cadlang(f_name)
        mcla.unfolder.set_stats()
        query = MCLSimpleQuery(None, None, prop)
        cam_list = mcla.mac_search(query, 10)
        res = len(cam_list) == 8
        self.assert_(res, 'Incorrect number of cam')
        list_expected = [['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'importinbeta', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'importinalpha', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'NUP214', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'MIZ_1', 'DAB2', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'NUP153', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'importinbeta', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'importinalpha', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'NUP214', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'DAB2', 'CEBPB', 'TGFBfamily__dimer___active_exCellRegion', 'TGFBR2__dimer___active_intToMb', 'Axin_SMAD3', 'SMAD4_cy', 'SMAD2_3', 'NUP153', 'FKBP12_TGFBR1__dimer___inactive_intToMb', 'CTGF', 'FOXG1']
                         ]
        list_obtained = []
        for cam in cam_list:
            list_obtained.append(cam.activated_frontier)
        res = lit_sol_equal(list_expected, list_obtained)
        self.assert_(res, 'Incorrect list of MAC (no clock)')


    def test_cam_clock(self):
        """
        cam with clocks
        """
        # solving
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        prop = 'p15INK4b'
        #p = "C3"
        f_name = "../../guard_transitions/translators/tests/tgf_cano_cmp.cal"
        mcla.build_from_cadlang(f_name)
        #mcla.build_from_cadlang("examples/delay2.cal")
        #mcla.unfolder.set_stats()
        query = MCLSimpleQuery(None, None, prop)
        cam_list = mcla.mac_search(query, 10)
        list_expected = [['SMAD4_nucl', 'FOXO1_3a_4_active_nucl', 'CEBPB', 'TGFB_TGFBR2_TGFBR1_DAB2_active_intToMb', 'SMAD2_3', 'FOXG1'],
                         ['SP1', 'SMAD4_nucl', 'MIZ_1', 'TGFB_TGFBR2_TGFBR1_DAB2_active_intToMb', 'SMAD2_3'],
                         ['SP1', 'betaglycan__dimer___intToMb', 'SMAD4_nucl', 'Caveolin_1', 'MIZ_1', 'TRAP_1_TGFBR1__dimer___inactive_intToMb', 'AP2B2', 'TGFBR2__dimer___active_intToMb', 'PML_SARA_SMAD2_3_earlyendosome', 'TGFBfamily__dimer___active_exCellRegion', 'SMAD7_SMURF1_2_nucl', 'CTGF'],
                         ['betaglycan__dimer___intToMb', 'SMAD4_nucl', 'Caveolin_1', 'FOXO1_3a_4_active_nucl', 'TRAP_1_TGFBR1__dimer___inactive_intToMb', 'AP2B2', 'CEBPB', 'TGFBR2__dimer___active_intToMb', 'PML_SARA_SMAD2_3_earlyendosome', 'TGFBfamily__dimer___active_exCellRegion', 'SMAD7_SMURF1_2_nucl', 'CTGF', 'FOXG1']
                         ]
        list_obtained = []
        for cam in cam_list:
            list_obtained.append(cam.activated_frontier)
        res = lit_sol_equal(list_expected, list_obtained)
        self.assert_(res, 'Incorrect list of MAC (clock)')




    def test_valid_simul_delay(self):
        """
        Test simulation of the delay example
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay1.cal")

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and  C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        # lsol: list<RawSolution>
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        f_sol = FrontierSolution.from_raw(sol)

        # independant simulation
        chm = ChartModel(" ")
        chm.build_from_cadlang("examples/delay1.cal", rep)
        sim = ChartSimulator()
        sim.build(chm, True, rep)
        sim.simul_init_places(f_sol.activated_frontier)
        sim.set_act_input_stream(f_sol.ic_sequence)
        sim.set_reachable_prop(f_prop)

        while True:
            try:
                sim.simul_step()
            except Exception, exc:
                exc_txt = exc.__str__()
                self.assert_(exc_txt == 'STOP', "Error: " + exc_txt)
                break
        # test if property reached
        res = sim.check_reachable_property()
        self.assert_(res, "Property not reached")


    def test_valid_simul_input(self):
        """
        Test simulation of the input example
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/input1.cal")

        st_prop = None
        inv_prop = None
        f_prop =  "C3 and  C4"
        query = MCLSimpleQuery(st_prop, inv_prop, f_prop)
        lsol = mcla.sq_solutions(query, 4, 1, mcla.unfolder.get_frontier())
        sol = lsol[0]
        f_sol = FrontierSolution.from_raw(sol)

        # independant simulation
        chm = ChartModel(" ")
        chm.build_from_cadlang("examples/input1.cal", rep)
        sim = ChartSimulator()
        sim.build(chm, True, rep)
        sim.simul_init_places(f_sol.activated_frontier)
        sim.set_act_input_stream(f_sol.ic_sequence)
        sim.set_reachable_prop(f_prop)

        while True:
            try:
                sim.simul_step()
            except Exception, exc:
                exc_txt = exc.__str__()
                self.assert_(exc_txt == 'STOP', "Error: " + exc_txt)
                break
        # test if property reached
        res = sim.check_reachable_property()
        self.assert_(res, "Property not reached")


    def test_cam_delay_save(self):
        """
        Test save on the delay example
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay1.cal")

        f_prop =  "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        macwic = mcla.mac_search(query, 10)

        f_out = file("x_file", 'w')
#        print macwic[0].activated_frontier
#        print macwic[0].ic_sequence
#        print '---------------'

        macwic[0].save(f_out)
        r_front = FrontierSolution.from_file("x_file")
#        print r_front.activated_frontier
#        print r_front.ic_sequence
        res = r_front.has_same_frontier(macwic[0])
        self.assert_(res, "Frontier solutions differ after read")
        os.remove("x_file")


    def test_cam_inhibitor(self):
        """
        test
        """
        rep = ErrorRep()
        mcla = MCLAnalyser(rep)
        mcla.build_from_cadlang("examples/delay3.cal")

        f_prop =  "C3 and C4"
        query = MCLSimpleQuery(None, None, f_prop)
        macwic = mcla.mac_search(query, 10)
        exp_act_front = ['A1', 'C1', 'B1']
        res = lit_subsol_equal(exp_act_front, macwic[0].activated_frontier)
        self.assert_(res, "Bad cam frontier solution")
        exp_timing = ["%", "% h2", "%", "%"]
        obt_timing = []
        obt_timing.extend(macwic[0].ic_sequence)
        res = lit_subsol_equal(exp_timing, obt_timing)
        self.assert_(res, "Bad cam timing solution")
        res = mcla.is_strong_activator(macwic[0], query)
        # it is not a strong activator
        self.assert_(not res, "Not a strong activator")
        inhib = mcla.mac_inhibitor_search(macwic[0], query, 10)
#        print 'INHIB:'
#        for isol in inhib:
#            print isol

        exp_act_front1 = ['D', 'A1', 'C1', 'B1']
        obt_act_front = inhib[0].activated_frontier
        obt_timing = []
        obt_timing.extend(inhib[0].ic_sequence)
        res = lit_subsol_equal(exp_act_front1, obt_act_front)
        if res:
            exp_timing = ["%", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")
        else:
            res = lit_subsol_equal(exp_act_front, obt_act_front)
            exp_timing = ["% h2", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")

        obt_act_front = inhib[1].activated_frontier
        obt_timing = []
        obt_timing.extend(inhib[1].ic_sequence)
        res = lit_subsol_equal(exp_act_front, obt_act_front)
        if res:
            exp_timing = ["% h2", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")
        else:
            res = lit_subsol_equal(exp_act_front1, obt_act_front)
            exp_timing = ["%", "% h2", "%", "%"]
            res = res and lit_subsol_equal(exp_timing, obt_timing)
            self.assert_(res, "Bad inhib timing solution")

        res = mcla.is_strong_inhibitor(inhib[0], query)
        # it is a strong inhibitor
        self.assert_(res, "Bad answer - strong inhibitor")





# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g 2012-09-11 14:55:48

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset


# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
F=12
WS=13
LETTER=15
OR=5
DOL=4
T=11
IDENT=8
NOT=7
DIGIT=16
AND=6
PD=10
COMMENT=14
EOF=-1
PG=9


class condexpLexer(Lexer):

    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"

    def __init__(self, input=None, state=None):
        if state is None:
            state = RecognizerSharedState()
        Lexer.__init__(self, input, state)

        self.dfa3 = self.DFA3(
            self, 3,
            eot = self.DFA3_eot,
            eof = self.DFA3_eof,
            min = self.DFA3_min,
            max = self.DFA3_max,
            accept = self.DFA3_accept,
            special = self.DFA3_special,
            transition = self.DFA3_transition
            )






    # $ANTLR start "WS"
    def mWS(self, ):

        try:
            _type = WS
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:55:13: ( ( ' ' | '\\t' | '\\n' ) )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:55:16: ( ' ' | '\\t' | '\\n' )
            pass
            if (9 <= self.input.LA(1) <= 10) or self.input.LA(1) == 32:
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse

            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "WS"



    # $ANTLR start "COMMENT"
    def mCOMMENT(self, ):

        try:
            _type = COMMENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:57:13: ( '//' (~ '\\n' )* '\\n' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:57:15: '//' (~ '\\n' )* '\\n'
            pass
            self.match("//")
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:57:19: (~ '\\n' )*
            while True: #loop1
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if ((0 <= LA1_0 <= 9) or (11 <= LA1_0 <= 65535)) :
                    alt1 = 1


                if alt1 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:57:20: ~ '\\n'
                    pass
                    if (0 <= self.input.LA(1) <= 9) or (11 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    break #loop1


            self.match(10)
            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COMMENT"



    # $ANTLR start "AND"
    def mAND(self, ):

        try:
            _type = AND
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:60:9: ( 'and' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:60:11: 'and'
            pass
            self.match("and")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "AND"



    # $ANTLR start "OR"
    def mOR(self, ):

        try:
            _type = OR
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:61:9: ( 'or' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:61:11: 'or'
            pass
            self.match("or")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "OR"



    # $ANTLR start "NOT"
    def mNOT(self, ):

        try:
            _type = NOT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:62:9: ( 'not' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:62:11: 'not'
            pass
            self.match("not")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "NOT"



    # $ANTLR start "T"
    def mT(self, ):

        try:
            _type = T
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:63:9: ( 'true' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:63:11: 'true'
            pass
            self.match("true")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "T"



    # $ANTLR start "F"
    def mF(self, ):

        try:
            _type = F
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:64:9: ( 'false' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:64:11: 'false'
            pass
            self.match("false")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "F"



    # $ANTLR start "PG"
    def mPG(self, ):

        try:
            _type = PG
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:66:9: ( '(' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:66:11: '('
            pass
            self.match(40)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "PG"



    # $ANTLR start "PD"
    def mPD(self, ):

        try:
            _type = PD
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:67:9: ( ')' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:67:11: ')'
            pass
            self.match(41)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "PD"



    # $ANTLR start "DOL"
    def mDOL(self, ):

        try:
            _type = DOL
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:68:9: ( '$' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:68:11: '$'
            pass
            self.match(36)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "DOL"



    # $ANTLR start "LETTER"
    def mLETTER(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:71:19: ( 'a' .. 'z' | 'A' .. 'Z' | '_' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:
            pass
            if (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse





        finally:

            pass

    # $ANTLR end "LETTER"



    # $ANTLR start "DIGIT"
    def mDIGIT(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:73:19: ( '0' .. '9' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:73:21: '0' .. '9'
            pass
            self.matchRange(48, 57)




        finally:

            pass

    # $ANTLR end "DIGIT"



    # $ANTLR start "IDENT"
    def mIDENT(self, ):

        try:
            _type = IDENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:75:9: ( ( LETTER | DIGIT )+ )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:75:11: ( LETTER | DIGIT )+
            pass
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:75:11: ( LETTER | DIGIT )+
            cnt2 = 0
            while True: #loop2
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if ((48 <= LA2_0 <= 57) or (65 <= LA2_0 <= 90) or LA2_0 == 95 or (97 <= LA2_0 <= 122)) :
                    alt2 = 1


                if alt2 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:
                    pass
                    if (48 <= self.input.LA(1) <= 57) or (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    if cnt2 >= 1:
                        break #loop2

                    eee = EarlyExitException(2, self.input)
                    raise eee

                cnt2 += 1





            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "IDENT"



    def mTokens(self):
        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:8: ( WS | COMMENT | AND | OR | NOT | T | F | PG | PD | DOL | IDENT )
        alt3 = 11
        alt3 = self.dfa3.predict(self.input)
        if alt3 == 1:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:10: WS
            pass
            self.mWS()


        elif alt3 == 2:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:13: COMMENT
            pass
            self.mCOMMENT()


        elif alt3 == 3:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:21: AND
            pass
            self.mAND()


        elif alt3 == 4:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:25: OR
            pass
            self.mOR()


        elif alt3 == 5:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:28: NOT
            pass
            self.mNOT()


        elif alt3 == 6:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:32: T
            pass
            self.mT()


        elif alt3 == 7:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:34: F
            pass
            self.mF()


        elif alt3 == 8:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:36: PG
            pass
            self.mPG()


        elif alt3 == 9:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:39: PD
            pass
            self.mPD()


        elif alt3 == 10:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:42: DOL
            pass
            self.mDOL()


        elif alt3 == 11:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/condexp.g:1:46: IDENT
            pass
            self.mIDENT()







    # lookup tables for DFA #3

    DFA3_eot = DFA.unpack(
        u"\3\uffff\5\13\4\uffff\1\13\1\22\3\13\1\26\1\uffff\1\27\2\13\2\uffff"
        u"\1\32\1\13\1\uffff\1\34\1\uffff"
        )

    DFA3_eof = DFA.unpack(
        u"\35\uffff"
        )

    DFA3_min = DFA.unpack(
        u"\1\11\2\uffff\1\156\1\162\1\157\1\162\1\141\4\uffff\1\144\1\60"
        u"\1\164\1\165\1\154\1\60\1\uffff\1\60\1\145\1\163\2\uffff\1\60\1"
        u"\145\1\uffff\1\60\1\uffff"
        )

    DFA3_max = DFA.unpack(
        u"\1\172\2\uffff\1\156\1\162\1\157\1\162\1\141\4\uffff\1\144\1\172"
        u"\1\164\1\165\1\154\1\172\1\uffff\1\172\1\145\1\163\2\uffff\1\172"
        u"\1\145\1\uffff\1\172\1\uffff"
        )

    DFA3_accept = DFA.unpack(
        u"\1\uffff\1\1\1\2\5\uffff\1\10\1\11\1\12\1\13\6\uffff\1\4\3\uffff"
        u"\1\3\1\5\2\uffff\1\6\1\uffff\1\7"
        )

    DFA3_special = DFA.unpack(
        u"\35\uffff"
        )


    DFA3_transition = [
        DFA.unpack(u"\2\1\25\uffff\1\1\3\uffff\1\12\3\uffff\1\10\1\11\5\uffff"
        u"\1\2\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\1\3\4\13\1\7\7\13"
        u"\1\5\1\4\4\13\1\6\6\13"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\14"),
        DFA.unpack(u"\1\15"),
        DFA.unpack(u"\1\16"),
        DFA.unpack(u"\1\17"),
        DFA.unpack(u"\1\20"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\21"),
        DFA.unpack(u"\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\32\13"),
        DFA.unpack(u"\1\23"),
        DFA.unpack(u"\1\24"),
        DFA.unpack(u"\1\25"),
        DFA.unpack(u"\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\32\13"),
        DFA.unpack(u""),
        DFA.unpack(u"\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\32\13"),
        DFA.unpack(u"\1\30"),
        DFA.unpack(u"\1\31"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\32\13"),
        DFA.unpack(u"\1\33"),
        DFA.unpack(u""),
        DFA.unpack(u"\12\13\7\uffff\32\13\4\uffff\1\13\1\uffff\32\13"),
        DFA.unpack(u"")
    ]

    # class definition for DFA #3

    DFA3 = DFA




def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import LexerMain
    main = LexerMain(condexpLexer)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)



# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g 2012-09-11 14:55:44

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset

from cadbiom.models.guard_transitions.chart_model import ChartModel



# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
RB=15
RP=11
LETTER=21
LP=9
COOP=8
INT=6
NOT=18
AND=16
EOF=-1
SC=14
WS=19
COM=10
OR=17
PROC=4
ARROW=7
IDENT=5
LB=13
IN_KW=12
DIGIT=22
COMMENT=20

# token names
tokenNames = [
    "<invalid>", "<EOR>", "<DOWN>", "<UP>",
    "PROC", "IDENT", "INT", "ARROW", "COOP", "LP", "COM", "RP", "IN_KW",
    "LB", "SC", "RB", "AND", "OR", "NOT", "WS", "COMMENT", "LETTER", "DIGIT"
]




class pintlangParser(Parser):
    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"
    tokenNames = tokenNames

    def __init__(self, input, state=None):
        if state is None:
            state = RecognizerSharedState()

        Parser.__init__(self, input, state)













    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
        hdr = self.getErrorHeader(re)
        msg = self.getErrorMessage(re, tokenNames)
        self.error_reporter.display(hdr,msg)


    def displayExceptionMessage(self, e):
        msg = self.getErrorMessage(self, e, tokenNames)
        self.error_reporter.display('',msg)

    def check_process(self, name, level, line):
        line_txt = 'line '+str(line)+':'
        try:
            lev = self.symb_tab[name]
            self.error_reporter.display(line_txt, "Process double declaration: "+id)

        except KeyError:
            self.symb_tab[name] = level
            for ii in range(level+1):
                pname = name + '_' + str(ii)
                self.model.get_root().add_simple_node(pname, 0.0, 0.0)

    def check_action(self, fname, flevel, cname, clevel1, clevel2, line):
        line_txt = 'line '+str(line)+':'
        try:
            lev = self.symb_tab[fname]
            if flevel>lev:
                self.error_reporter(line_txt, "Incorrect level for:"+fname)
                return
        except KeyError:
            self.error_reporter.display(line_txt, "Undeclared process: "+fname)
            return
        try:
            lev = self.symb_tab[cname]
            if clevel1>lev or clevel2>lev:
                self.error_reporter(line_txt, "Incorrect level for:"+cname)
                return
        except KeyError:
            self.error_reporter.display(line_txt, "Undeclared process: "+cname)
            return

        n1 = self.model.get_simple_node(cname+'_'+str(clevel1))
        n2 = self.model.get_simple_node(cname+'_'+str(clevel2))
        tr=self.model.get_root().add_transition(n1, n2)
        tr.set_condition(fname+'_'+str(flevel))
        h_name = 'hh_'+str(self.clock_cpt)
        self.clock_cpt +=1
        tr.set_event(h_name)
        self.clock_list.append(h_name)
        return

    def check_transition(self, cname, clevel1, clevel2, line):
        line_txt = 'line '+str(line)+':'
        try:
            lev = self.symb_tab[cname]
            if clevel1>lev or clevel2>lev:
                self.error_reporter(line_txt, "Incorrect level for:"+cname)
                return
        except KeyError:
            self.error_reporter.display(line_txt, "Undeclared process: "+cname)
            return

        n1 = self.model.get_simple_node(cname+'_'+str(clevel1))
        n2 = self.model.get_simple_node(cname+'_'+str(clevel2))
        tr=self.model.get_root().add_transition(n1, n2)
        return tr

    def check_tt_rhs(self, tr, il ,l_cond):
        line_txt = 'line '+'? :'
        cond = self.tt_translate_lcond(il, l_cond, line_txt)
        tr.set_condition(cond)
        h_name = 'hh_'+str(self.clock_cpt)
        self.clock_cpt +=1
        tr.set_event(h_name)
        self.clock_list.append(h_name)

    def check_tr_rhs(self, cond, cname, clevel1, clevel2, line):
        line_txt = 'line '+str(line)+':'
        try:
            lev = self.symb_tab[cname]
            if clevel1>lev or clevel2>lev:
                self.error_reporter(line_txt, "Incorrect level for:"+cname)
                return
        except KeyError:
            self.error_reporter.display(line_txt, "Undeclared process: "+cname)
            return

        n1 = self.model.get_simple_node(cname+'_'+str(clevel1))
        n2 = self.model.get_simple_node(cname+'_'+str(clevel2))
        tr=self.model.get_root().add_transition(n1, n2)
        tr.set_condition(cond)
        h_name = 'hh_'+str(self.clock_cpt)
        self.clock_cpt +=1
        tr.set_event(h_name)
        self.clock_list.append(h_name)


    def tt_translate_cond(self, id_list, val_list, line_txt):
        if len(id_list) != len(val_list):
            self.error_reporter.display(line_txt, "Bad condition specification"+id_list.__str__())
            return None
        cond = id_list[0] + "_"+str(val_list[0])
        for i in range(1, len(id_list)):
            cond_el = id_list[i] + "_" + str(val_list[i])
            cond = cond + " and "+ cond_el
        return cond


    def tt_translate_lcond(self, id_list, lval_list, line_txt):
        cond_el = self.tt_translate_cond(id_list, lval_list[0], line_txt)
        if cond_el:
            cond = '(' + cond_el + ')'
        else:
            return ""
        for i in range(1,len(lval_list)):
            cond_el = self.tt_translate_cond(id_list, lval_list[i], line_txt)
            if cond_el:
                cond = cond + ' or (' + cond_el + ')'
            else:
                return ""
        return cond





    # $ANTLR start "pintspec"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:158:1: pintspec[model_name] : ( process | action | cooper )* EOF ;
    def pintspec(self, model_name):


        self.model = ChartModel(model_name)
        self.symb_tab = dict()
        self.clock_cpt = 0
        self.clock_list = []
        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:12: ( ( process | action | cooper )* EOF )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:14: ( process | action | cooper )* EOF
                pass
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:14: ( process | action | cooper )*
                while True: #loop1
                    alt1 = 4
                    LA1 = self.input.LA(1)
                    if LA1 == PROC:
                        alt1 = 1
                    elif LA1 == IDENT:
                        alt1 = 2
                    elif LA1 == COOP:
                        alt1 = 3

                    if alt1 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:15: process
                        pass
                        self._state.following.append(self.FOLLOW_process_in_pintspec72)
                        self.process()

                        self._state.following.pop()


                    elif alt1 == 2:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:23: action
                        pass
                        self._state.following.append(self.FOLLOW_action_in_pintspec74)
                        self.action()

                        self._state.following.pop()


                    elif alt1 == 3:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:163:30: cooper
                        pass
                        self._state.following.append(self.FOLLOW_cooper_in_pintspec76)
                        self.cooper()

                        self._state.following.pop()


                    else:
                        break #loop1


                self.match(self.input, EOF, self.FOLLOW_EOF_in_pintspec81)




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return

    # $ANTLR end "pintspec"


    # $ANTLR start "process"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:166:1: process : PROC id= IDENT i= INT ;
    def process(self, ):

        id = None
        i = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:167:12: ( PROC id= IDENT i= INT )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:167:14: PROC id= IDENT i= INT
                pass
                self.match(self.input, PROC, self.FOLLOW_PROC_in_process123)
                id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_process127)
                i=self.match(self.input, INT, self.FOLLOW_INT_in_process132)
                #action start
                self.check_process(id.text, int(i.text), id.line)
                #action end




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return

    # $ANTLR end "process"


    # $ANTLR start "action"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:171:1: action : fid= IDENT flev= INT ARROW cid= IDENT lev1= INT lev2= INT ;
    def action(self, ):

        fid = None
        flev = None
        cid = None
        lev1 = None
        lev2 = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:172:12: (fid= IDENT flev= INT ARROW cid= IDENT lev1= INT lev2= INT )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:172:14: fid= IDENT flev= INT ARROW cid= IDENT lev1= INT lev2= INT
                pass
                fid=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_action194)
                flev=self.match(self.input, INT, self.FOLLOW_INT_in_action198)
                self.match(self.input, ARROW, self.FOLLOW_ARROW_in_action200)
                cid=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_action204)
                lev1=self.match(self.input, INT, self.FOLLOW_INT_in_action208)
                lev2=self.match(self.input, INT, self.FOLLOW_INT_in_action212)
                #action start
                self.check_action(fid.text, int(flev.text), cid.text, int(lev1.text), int(lev2.text), fid.line)
                #action end




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return

    # $ANTLR end "action"


    # $ANTLR start "cooper"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:176:1: cooper : COOP LP lha= coop_lhs COM coop_rhs[$lha.transition, $lha.id_list, $lha.condition] RP ;
    def cooper(self, ):

        lha = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:177:12: ( COOP LP lha= coop_lhs COM coop_rhs[$lha.transition, $lha.id_list, $lha.condition] RP )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:177:14: COOP LP lha= coop_lhs COM coop_rhs[$lha.transition, $lha.id_list, $lha.condition] RP
                pass
                self.match(self.input, COOP, self.FOLLOW_COOP_in_cooper275)
                self.match(self.input, LP, self.FOLLOW_LP_in_cooper277)
                self._state.following.append(self.FOLLOW_coop_lhs_in_cooper281)
                lha = self.coop_lhs()

                self._state.following.pop()
                self.match(self.input, COM, self.FOLLOW_COM_in_cooper283)
                self._state.following.append(self.FOLLOW_coop_rhs_in_cooper285)
                self.coop_rhs(((lha is not None) and [lha.transition] or [None])[0], ((lha is not None) and [lha.id_list] or [None])[0], ((lha is not None) and [lha.condition] or [None])[0])

                self._state.following.pop()
                self.match(self.input, RP, self.FOLLOW_RP_in_cooper288)




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return

    # $ANTLR end "cooper"

    class coop_lhs_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.transition = None
            self.id_list = None
            self.condition = None




    # $ANTLR start "coop_lhs"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:180:1: coop_lhs returns [transition, id_list, condition] : il= sort_list rc= rcoop_lhs[il] ;
    def coop_lhs(self, ):

        retval = self.coop_lhs_return()
        retval.start = self.input.LT(1)

        il = None

        rc = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:181:12: (il= sort_list rc= rcoop_lhs[il] )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:181:14: il= sort_list rc= rcoop_lhs[il]
                pass
                self._state.following.append(self.FOLLOW_sort_list_in_coop_lhs338)
                il = self.sort_list()

                self._state.following.pop()
                #action start
                retval.id_list = il
                #action end
                self._state.following.append(self.FOLLOW_rcoop_lhs_in_coop_lhs372)
                rc = self.rcoop_lhs(il)

                self._state.following.pop()
                #action start
                retval.transition = ((rc is not None) and [rc.transition] or [None])[0]
                retval.condition = ((rc is not None) and [rc.condition] or [None])[0]
                #action end



                retval.stop = self.input.LT(-1)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return retval

    # $ANTLR end "coop_lhs"

    class rcoop_lhs_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.transition = None
            self.condition = None




    # $ANTLR start "rcoop_lhs"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:188:1: rcoop_lhs[ids] returns [transition, condition] : (kw= IN_KW tt= tt_condition cond= tail_logexp | ARROW id= IDENT i1= INT i2= INT );
    def rcoop_lhs(self, ids):

        retval = self.rcoop_lhs_return()
        retval.start = self.input.LT(1)

        kw = None
        id = None
        i1 = None
        i2 = None
        tt = None

        cond = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:189:12: (kw= IN_KW tt= tt_condition cond= tail_logexp | ARROW id= IDENT i1= INT i2= INT )
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if (LA2_0 == IN_KW) :
                    alt2 = 1
                elif (LA2_0 == ARROW) :
                    alt2 = 2
                else:
                    nvae = NoViableAltException("", 2, 0, self.input)

                    raise nvae

                if alt2 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:189:14: kw= IN_KW tt= tt_condition cond= tail_logexp
                    pass
                    kw=self.match(self.input, IN_KW, self.FOLLOW_IN_KW_in_rcoop_lhs438)
                    self._state.following.append(self.FOLLOW_tt_condition_in_rcoop_lhs443)
                    tt = self.tt_condition()

                    self._state.following.pop()
                    self._state.following.append(self.FOLLOW_tail_logexp_in_rcoop_lhs447)
                    cond = self.tail_logexp()

                    self._state.following.pop()
                    #action start
                    retval.condition = self.tt_translate_lcond(ids, tt,  'line '+str(kw.line)+':') + cond
                    #action end


                elif alt2 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:191:14: ARROW id= IDENT i1= INT i2= INT
                    pass
                    self.match(self.input, ARROW, self.FOLLOW_ARROW_in_rcoop_lhs478)
                    id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_rcoop_lhs482)
                    i1=self.match(self.input, INT, self.FOLLOW_INT_in_rcoop_lhs486)
                    i2=self.match(self.input, INT, self.FOLLOW_INT_in_rcoop_lhs490)
                    #action start
                    retval.transition = self.check_transition(id.text, int(i1.text), int(i2.text), id.line)
                    #action end


                retval.stop = self.input.LT(-1)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return retval

    # $ANTLR end "rcoop_lhs"


    # $ANTLR start "coop_rhs"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:195:1: coop_rhs[tr, il, cond] : (tt= tt_condition | id= IDENT cc= COM i1= INT COM i2= INT );
    def coop_rhs(self, tr, il, cond):

        id = None
        cc = None
        i1 = None
        i2 = None
        tt = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:196:12: (tt= tt_condition | id= IDENT cc= COM i1= INT COM i2= INT )
                alt3 = 2
                LA3_0 = self.input.LA(1)

                if (LA3_0 == LB) :
                    alt3 = 1
                elif (LA3_0 == IDENT) :
                    alt3 = 2
                else:
                    nvae = NoViableAltException("", 3, 0, self.input)

                    raise nvae

                if alt3 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:196:14: tt= tt_condition
                    pass
                    self._state.following.append(self.FOLLOW_tt_condition_in_coop_rhs553)
                    tt = self.tt_condition()

                    self._state.following.pop()
                    #action start
                    self.check_tt_rhs(tr,il, tt, )
                    #action end


                elif alt3 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:198:14: id= IDENT cc= COM i1= INT COM i2= INT
                    pass
                    id=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_coop_rhs584)
                    cc=self.match(self.input, COM, self.FOLLOW_COM_in_coop_rhs588)
                    i1=self.match(self.input, INT, self.FOLLOW_INT_in_coop_rhs592)
                    self.match(self.input, COM, self.FOLLOW_COM_in_coop_rhs595)
                    i2=self.match(self.input, INT, self.FOLLOW_INT_in_coop_rhs599)
                    #action start
                    self.check_tr_rhs(cond, id.text, int(i1.text), int(i2.text), cc.line)
                    #action end



            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return

    # $ANTLR end "coop_rhs"


    # $ANTLR start "sort_list"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:202:1: sort_list returns [id_list] : LB id1= IDENT ( SC id2= IDENT )* RB ;
    def sort_list(self, ):

        id_list = None

        id1 = None
        id2 = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:203:12: ( LB id1= IDENT ( SC id2= IDENT )* RB )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:203:14: LB id1= IDENT ( SC id2= IDENT )* RB
                pass
                self.match(self.input, LB, self.FOLLOW_LB_in_sort_list660)
                id1=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sort_list664)
                #action start
                id_list = [id1.text]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:205:14: ( SC id2= IDENT )*
                while True: #loop4
                    alt4 = 2
                    LA4_0 = self.input.LA(1)

                    if (LA4_0 == SC) :
                        alt4 = 1


                    if alt4 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:205:15: SC id2= IDENT
                        pass
                        self.match(self.input, SC, self.FOLLOW_SC_in_sort_list699)
                        id2=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sort_list703)
                        #action start
                        id_list.append(id2.text)
                        #action end


                    else:
                        break #loop4


                self.match(self.input, RB, self.FOLLOW_RB_in_sort_list709)




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return id_list

    # $ANTLR end "sort_list"


    # $ANTLR start "tt_condition"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:208:1: tt_condition returns [l_cond] : LB tt1= tt_conjunction ( SC tt2= tt_conjunction )* RB ;
    def tt_condition(self, ):

        l_cond = None

        tt1 = None

        tt2 = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:209:12: ( LB tt1= tt_conjunction ( SC tt2= tt_conjunction )* RB )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:209:14: LB tt1= tt_conjunction ( SC tt2= tt_conjunction )* RB
                pass
                self.match(self.input, LB, self.FOLLOW_LB_in_tt_condition755)
                self._state.following.append(self.FOLLOW_tt_conjunction_in_tt_condition759)
                tt1 = self.tt_conjunction()

                self._state.following.pop()
                #action start
                l_cond = [tt1]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:210:14: ( SC tt2= tt_conjunction )*
                while True: #loop5
                    alt5 = 2
                    LA5_0 = self.input.LA(1)

                    if (LA5_0 == SC) :
                        alt5 = 1


                    if alt5 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:210:15: SC tt2= tt_conjunction
                        pass
                        self.match(self.input, SC, self.FOLLOW_SC_in_tt_condition777)
                        self._state.following.append(self.FOLLOW_tt_conjunction_in_tt_condition781)
                        tt2 = self.tt_conjunction()

                        self._state.following.pop()
                        #action start
                        l_cond.append(tt2)
                        #action end


                    else:
                        break #loop5


                self.match(self.input, RB, self.FOLLOW_RB_in_tt_condition788)




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return l_cond

    # $ANTLR end "tt_condition"


    # $ANTLR start "tt_conjunction"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:213:1: tt_conjunction returns [lval] : LB i1= INT ( SC i2= INT )* RB ;
    def tt_conjunction(self, ):

        lval = None

        i1 = None
        i2 = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:214:12: ( LB i1= INT ( SC i2= INT )* RB )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:214:15: LB i1= INT ( SC i2= INT )* RB
                pass
                self.match(self.input, LB, self.FOLLOW_LB_in_tt_conjunction835)
                i1=self.match(self.input, INT, self.FOLLOW_INT_in_tt_conjunction839)
                #action start
                lval = [int(i1.text)]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:215:15: ( SC i2= INT )*
                while True: #loop6
                    alt6 = 2
                    LA6_0 = self.input.LA(1)

                    if (LA6_0 == SC) :
                        alt6 = 1


                    if alt6 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:215:16: SC i2= INT
                        pass
                        self.match(self.input, SC, self.FOLLOW_SC_in_tt_conjunction858)
                        i2=self.match(self.input, INT, self.FOLLOW_INT_in_tt_conjunction862)
                        #action start
                        lval.append(int(i2.text))
                        #action end


                    else:
                        break #loop6


                self.match(self.input, RB, self.FOLLOW_RB_in_tt_conjunction869)




            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return lval

    # $ANTLR end "tt_conjunction"


    # $ANTLR start "tail_logexp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:218:1: tail_logexp returns [cond] : ( | AND tt1= tt_logexp | OR tt2= tt_logexp | tt_primary );
    def tail_logexp(self, ):

        cond = None

        tt1 = None

        tt2 = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:219:12: ( | AND tt1= tt_logexp | OR tt2= tt_logexp | tt_primary )
                alt7 = 4
                LA7 = self.input.LA(1)
                if LA7 == COM:
                    alt7 = 1
                elif LA7 == AND:
                    alt7 = 2
                elif LA7 == OR:
                    alt7 = 3
                elif LA7 == LP or LA7 == LB or LA7 == NOT:
                    alt7 = 4
                else:
                    nvae = NoViableAltException("", 7, 0, self.input)

                    raise nvae

                if alt7 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:219:14:
                    pass
                    #action start
                    cond = ""
                    #action end


                elif alt7 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:220:14: AND tt1= tt_logexp
                    pass
                    self.match(self.input, AND, self.FOLLOW_AND_in_tail_logexp931)
                    self._state.following.append(self.FOLLOW_tt_logexp_in_tail_logexp935)
                    tt1 = self.tt_logexp()

                    self._state.following.pop()
                    #action start
                    cond = ' and ' + tt1
                    #action end


                elif alt7 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:222:14: OR tt2= tt_logexp
                    pass
                    self.match(self.input, OR, self.FOLLOW_OR_in_tail_logexp966)
                    self._state.following.append(self.FOLLOW_tt_logexp_in_tail_logexp971)
                    tt2 = self.tt_logexp()

                    self._state.following.pop()
                    #action start
                    cond = ' and ' + tt1
                    #action end


                elif alt7 == 4:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:224:14: tt_primary
                    pass
                    self._state.following.append(self.FOLLOW_tt_primary_in_tail_logexp1002)
                    self.tt_primary()

                    self._state.following.pop()
                    #action start
                    cond = tt1
                    #action end



            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return cond

    # $ANTLR end "tail_logexp"


    # $ANTLR start "tt_logexp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:228:1: tt_logexp returns [cond] : tt1= tt_logexp2 ( OR tt2= tt_logexp2 )* ;
    def tt_logexp(self, ):

        cond = None

        tt1 = None

        tt2 = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:229:12: (tt1= tt_logexp2 ( OR tt2= tt_logexp2 )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:229:14: tt1= tt_logexp2 ( OR tt2= tt_logexp2 )*
                pass
                self._state.following.append(self.FOLLOW_tt_logexp2_in_tt_logexp1066)
                tt1 = self.tt_logexp2()

                self._state.following.pop()
                #action start
                cond = tt1
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:230:13: ( OR tt2= tt_logexp2 )*
                while True: #loop8
                    alt8 = 2
                    LA8_0 = self.input.LA(1)

                    if (LA8_0 == OR) :
                        alt8 = 1


                    if alt8 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:230:14: OR tt2= tt_logexp2
                        pass
                        self.match(self.input, OR, self.FOLLOW_OR_in_tt_logexp1083)
                        self._state.following.append(self.FOLLOW_tt_logexp2_in_tt_logexp1087)
                        tt2 = self.tt_logexp2()

                        self._state.following.pop()
                        #action start
                        cond = cond + ' or ' + tt2
                        #action end


                    else:
                        break #loop8






            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return cond

    # $ANTLR end "tt_logexp"


    # $ANTLR start "tt_logexp2"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:233:1: tt_logexp2 returns [cond] : tt1= tt_primary ( AND tt2= tt_primary )* ;
    def tt_logexp2(self, ):

        cond = None

        tt1 = None

        tt2 = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:234:12: (tt1= tt_primary ( AND tt2= tt_primary )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:234:14: tt1= tt_primary ( AND tt2= tt_primary )*
                pass
                self._state.following.append(self.FOLLOW_tt_primary_in_tt_logexp21139)
                tt1 = self.tt_primary()

                self._state.following.pop()
                #action start
                cond = tt1
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:235:14: ( AND tt2= tt_primary )*
                while True: #loop9
                    alt9 = 2
                    LA9_0 = self.input.LA(1)

                    if (LA9_0 == AND) :
                        alt9 = 1


                    if alt9 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:235:15: AND tt2= tt_primary
                        pass
                        self.match(self.input, AND, self.FOLLOW_AND_in_tt_logexp21157)
                        self._state.following.append(self.FOLLOW_tt_primary_in_tt_logexp21161)
                        tt2 = self.tt_primary()

                        self._state.following.pop()
                        #action start
                        cond = cond + ' and ' + tt2
                        #action end


                    else:
                        break #loop9






            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return cond

    # $ANTLR end "tt_logexp2"


    # $ANTLR start "tt_primary"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:238:1: tt_primary returns [cond] : ( NOT tt1= tt_primary | st= sort_list kw= IN_KW tt2= tt_condition | LP tt3= tt_logexp RP );
    def tt_primary(self, ):

        cond = None

        kw = None
        tt1 = None

        st = None

        tt2 = None

        tt3 = None


        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:239:12: ( NOT tt1= tt_primary | st= sort_list kw= IN_KW tt2= tt_condition | LP tt3= tt_logexp RP )
                alt10 = 3
                LA10 = self.input.LA(1)
                if LA10 == NOT:
                    alt10 = 1
                elif LA10 == LB:
                    alt10 = 2
                elif LA10 == LP:
                    alt10 = 3
                else:
                    nvae = NoViableAltException("", 10, 0, self.input)

                    raise nvae

                if alt10 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:239:14: NOT tt1= tt_primary
                    pass
                    self.match(self.input, NOT, self.FOLLOW_NOT_in_tt_primary1212)
                    self._state.following.append(self.FOLLOW_tt_primary_in_tt_primary1216)
                    tt1 = self.tt_primary()

                    self._state.following.pop()
                    #action start
                    cond = 'not (' + tt1 + ')'
                    #action end


                elif alt10 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:241:14: st= sort_list kw= IN_KW tt2= tt_condition
                    pass
                    self._state.following.append(self.FOLLOW_sort_list_in_tt_primary1247)
                    st = self.sort_list()

                    self._state.following.pop()
                    kw=self.match(self.input, IN_KW, self.FOLLOW_IN_KW_in_tt_primary1251)
                    self._state.following.append(self.FOLLOW_tt_condition_in_tt_primary1255)
                    tt2 = self.tt_condition()

                    self._state.following.pop()
                    #action start
                    cond = self.tt_translate_lcond(st, tt2, 'line '+str(kw.line)+':')
                    #action end


                elif alt10 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:243:14: LP tt3= tt_logexp RP
                    pass
                    self.match(self.input, LP, self.FOLLOW_LP_in_tt_primary1284)
                    self._state.following.append(self.FOLLOW_tt_logexp_in_tt_primary1288)
                    tt3 = self.tt_logexp()

                    self._state.following.pop()
                    self.match(self.input, RP, self.FOLLOW_RP_in_tt_primary1290)
                    #action start
                    cond = '('+tt3+')'
                    #action end



            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
        finally:

            pass

        return cond

    # $ANTLR end "tt_primary"


    # Delegated rules




    FOLLOW_process_in_pintspec72 = frozenset([4, 5, 8])
    FOLLOW_action_in_pintspec74 = frozenset([4, 5, 8])
    FOLLOW_cooper_in_pintspec76 = frozenset([4, 5, 8])
    FOLLOW_EOF_in_pintspec81 = frozenset([1])
    FOLLOW_PROC_in_process123 = frozenset([5])
    FOLLOW_IDENT_in_process127 = frozenset([6])
    FOLLOW_INT_in_process132 = frozenset([1])
    FOLLOW_IDENT_in_action194 = frozenset([6])
    FOLLOW_INT_in_action198 = frozenset([7])
    FOLLOW_ARROW_in_action200 = frozenset([5])
    FOLLOW_IDENT_in_action204 = frozenset([6])
    FOLLOW_INT_in_action208 = frozenset([6])
    FOLLOW_INT_in_action212 = frozenset([1])
    FOLLOW_COOP_in_cooper275 = frozenset([9])
    FOLLOW_LP_in_cooper277 = frozenset([13])
    FOLLOW_coop_lhs_in_cooper281 = frozenset([10])
    FOLLOW_COM_in_cooper283 = frozenset([5, 13])
    FOLLOW_coop_rhs_in_cooper285 = frozenset([11])
    FOLLOW_RP_in_cooper288 = frozenset([1])
    FOLLOW_sort_list_in_coop_lhs338 = frozenset([7, 12])
    FOLLOW_rcoop_lhs_in_coop_lhs372 = frozenset([1])
    FOLLOW_IN_KW_in_rcoop_lhs438 = frozenset([13])
    FOLLOW_tt_condition_in_rcoop_lhs443 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tail_logexp_in_rcoop_lhs447 = frozenset([1])
    FOLLOW_ARROW_in_rcoop_lhs478 = frozenset([5])
    FOLLOW_IDENT_in_rcoop_lhs482 = frozenset([6])
    FOLLOW_INT_in_rcoop_lhs486 = frozenset([6])
    FOLLOW_INT_in_rcoop_lhs490 = frozenset([1])
    FOLLOW_tt_condition_in_coop_rhs553 = frozenset([1])
    FOLLOW_IDENT_in_coop_rhs584 = frozenset([10])
    FOLLOW_COM_in_coop_rhs588 = frozenset([6])
    FOLLOW_INT_in_coop_rhs592 = frozenset([10])
    FOLLOW_COM_in_coop_rhs595 = frozenset([6])
    FOLLOW_INT_in_coop_rhs599 = frozenset([1])
    FOLLOW_LB_in_sort_list660 = frozenset([5])
    FOLLOW_IDENT_in_sort_list664 = frozenset([14, 15])
    FOLLOW_SC_in_sort_list699 = frozenset([5])
    FOLLOW_IDENT_in_sort_list703 = frozenset([14, 15])
    FOLLOW_RB_in_sort_list709 = frozenset([1])
    FOLLOW_LB_in_tt_condition755 = frozenset([13])
    FOLLOW_tt_conjunction_in_tt_condition759 = frozenset([14, 15])
    FOLLOW_SC_in_tt_condition777 = frozenset([13])
    FOLLOW_tt_conjunction_in_tt_condition781 = frozenset([14, 15])
    FOLLOW_RB_in_tt_condition788 = frozenset([1])
    FOLLOW_LB_in_tt_conjunction835 = frozenset([6])
    FOLLOW_INT_in_tt_conjunction839 = frozenset([14, 15])
    FOLLOW_SC_in_tt_conjunction858 = frozenset([6])
    FOLLOW_INT_in_tt_conjunction862 = frozenset([14, 15])
    FOLLOW_RB_in_tt_conjunction869 = frozenset([1])
    FOLLOW_AND_in_tail_logexp931 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_logexp_in_tail_logexp935 = frozenset([1])
    FOLLOW_OR_in_tail_logexp966 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_logexp_in_tail_logexp971 = frozenset([1])
    FOLLOW_tt_primary_in_tail_logexp1002 = frozenset([1])
    FOLLOW_tt_logexp2_in_tt_logexp1066 = frozenset([1, 17])
    FOLLOW_OR_in_tt_logexp1083 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_logexp2_in_tt_logexp1087 = frozenset([1, 17])
    FOLLOW_tt_primary_in_tt_logexp21139 = frozenset([1, 16])
    FOLLOW_AND_in_tt_logexp21157 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_primary_in_tt_logexp21161 = frozenset([1, 16])
    FOLLOW_NOT_in_tt_primary1212 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_primary_in_tt_primary1216 = frozenset([1])
    FOLLOW_sort_list_in_tt_primary1247 = frozenset([12])
    FOLLOW_IN_KW_in_tt_primary1251 = frozenset([13])
    FOLLOW_tt_condition_in_tt_primary1255 = frozenset([1])
    FOLLOW_LP_in_tt_primary1284 = frozenset([9, 13, 16, 17, 18])
    FOLLOW_tt_logexp_in_tt_primary1288 = frozenset([11])
    FOLLOW_RP_in_tt_primary1290 = frozenset([1])



def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import ParserMain
    main = ParserMain("pintlangLexer", pintlangParser)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)



# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g 2012-09-11 14:55:44

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset


# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
RB=15
RP=11
LETTER=21
LP=9
COOP=8
INT=6
NOT=18
AND=16
EOF=-1
SC=14
WS=19
COM=10
OR=17
PROC=4
ARROW=7
IDENT=5
LB=13
IN_KW=12
DIGIT=22
COMMENT=20


class pintlangLexer(Lexer):

    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"

    def __init__(self, input=None, state=None):
        if state is None:
            state = RecognizerSharedState()
        Lexer.__init__(self, input, state)

        self.dfa4 = self.DFA4(
            self, 4,
            eot = self.DFA4_eot,
            eof = self.DFA4_eof,
            min = self.DFA4_min,
            max = self.DFA4_max,
            accept = self.DFA4_accept,
            special = self.DFA4_special,
            transition = self.DFA4_transition
            )





    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
        hdr = self.getErrorHeader(re)
        msg = self.getErrorMessage(re, tokenNames)
        self.error_reporter.display(hdr,msg)


    def displayExceptionMessage(self, e):
        msg = self.getErrorMessage(self, e, tokenNames)
        self.error_reporter.display('',msg)



    # $ANTLR start "WS"
    def mWS(self, ):

        try:
            _type = WS
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:251:13: ( ( ' ' | '\\t' | '\\n' ) )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:251:16: ( ' ' | '\\t' | '\\n' )
            pass
            if (9 <= self.input.LA(1) <= 10) or self.input.LA(1) == 32:
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse

            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "WS"



    # $ANTLR start "COMMENT"
    def mCOMMENT(self, ):

        try:
            _type = COMMENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:253:13: ( '//' (~ '\\n' )* '\\n' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:253:15: '//' (~ '\\n' )* '\\n'
            pass
            self.match("//")
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:253:19: (~ '\\n' )*
            while True: #loop1
                alt1 = 2
                LA1_0 = self.input.LA(1)

                if ((0 <= LA1_0 <= 9) or (11 <= LA1_0 <= 65535)) :
                    alt1 = 1


                if alt1 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:253:20: ~ '\\n'
                    pass
                    if (0 <= self.input.LA(1) <= 9) or (11 <= self.input.LA(1) <= 65535):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    break #loop1


            self.match(10)
            #action start
            _channel = HIDDEN;
            #action end



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COMMENT"



    # $ANTLR start "PROC"
    def mPROC(self, ):

        try:
            _type = PROC
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:256:9: ( 'process' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:256:11: 'process'
            pass
            self.match("process")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "PROC"



    # $ANTLR start "ARROW"
    def mARROW(self, ):

        try:
            _type = ARROW
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:257:9: ( '->' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:257:11: '->'
            pass
            self.match("->")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "ARROW"



    # $ANTLR start "COOP"
    def mCOOP(self, ):

        try:
            _type = COOP
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:258:9: ( 'COOPERATIVITY' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:258:11: 'COOPERATIVITY'
            pass
            self.match("COOPERATIVITY")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COOP"



    # $ANTLR start "IN_KW"
    def mIN_KW(self, ):

        try:
            _type = IN_KW
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:259:9: ( 'in' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:259:11: 'in'
            pass
            self.match("in")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "IN_KW"



    # $ANTLR start "LP"
    def mLP(self, ):

        try:
            _type = LP
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:261:9: ( '(' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:261:11: '('
            pass
            self.match(40)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "LP"



    # $ANTLR start "RP"
    def mRP(self, ):

        try:
            _type = RP
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:262:9: ( ')' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:262:11: ')'
            pass
            self.match(41)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "RP"



    # $ANTLR start "LB"
    def mLB(self, ):

        try:
            _type = LB
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:263:9: ( '[' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:263:11: '['
            pass
            self.match(91)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "LB"



    # $ANTLR start "RB"
    def mRB(self, ):

        try:
            _type = RB
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:264:9: ( ']' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:264:11: ']'
            pass
            self.match(93)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "RB"



    # $ANTLR start "SC"
    def mSC(self, ):

        try:
            _type = SC
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:265:9: ( ';' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:265:11: ';'
            pass
            self.match(59)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "SC"



    # $ANTLR start "COM"
    def mCOM(self, ):

        try:
            _type = COM
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:266:9: ( ',' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:266:11: ','
            pass
            self.match(44)



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "COM"



    # $ANTLR start "AND"
    def mAND(self, ):

        try:
            _type = AND
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:268:9: ( 'and' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:268:11: 'and'
            pass
            self.match("and")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "AND"



    # $ANTLR start "OR"
    def mOR(self, ):

        try:
            _type = OR
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:269:9: ( 'or' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:269:11: 'or'
            pass
            self.match("or")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "OR"



    # $ANTLR start "NOT"
    def mNOT(self, ):

        try:
            _type = NOT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:270:9: ( 'not' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:270:11: 'not'
            pass
            self.match("not")



            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "NOT"



    # $ANTLR start "LETTER"
    def mLETTER(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:272:19: ( 'a' .. 'z' | 'A' .. 'Z' | '_' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:
            pass
            if (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                self.input.consume()
            else:
                mse = MismatchedSetException(None, self.input)
                self.recover(mse)
                raise mse





        finally:

            pass

    # $ANTLR end "LETTER"



    # $ANTLR start "DIGIT"
    def mDIGIT(self, ):

        try:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:274:19: ( '0' .. '9' )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:274:21: '0' .. '9'
            pass
            self.matchRange(48, 57)




        finally:

            pass

    # $ANTLR end "DIGIT"



    # $ANTLR start "IDENT"
    def mIDENT(self, ):

        try:
            _type = IDENT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:276:9: ( LETTER ( LETTER | DIGIT )* )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:276:11: LETTER ( LETTER | DIGIT )*
            pass
            self.mLETTER()
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:276:17: ( LETTER | DIGIT )*
            while True: #loop2
                alt2 = 2
                LA2_0 = self.input.LA(1)

                if ((48 <= LA2_0 <= 57) or (65 <= LA2_0 <= 90) or LA2_0 == 95 or (97 <= LA2_0 <= 122)) :
                    alt2 = 1


                if alt2 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:
                    pass
                    if (48 <= self.input.LA(1) <= 57) or (65 <= self.input.LA(1) <= 90) or self.input.LA(1) == 95 or (97 <= self.input.LA(1) <= 122):
                        self.input.consume()
                    else:
                        mse = MismatchedSetException(None, self.input)
                        self.recover(mse)
                        raise mse



                else:
                    break #loop2





            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "IDENT"



    # $ANTLR start "INT"
    def mINT(self, ):

        try:
            _type = INT
            _channel = DEFAULT_CHANNEL

            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:278:9: ( ( DIGIT )+ )
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:278:11: ( DIGIT )+
            pass
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:278:11: ( DIGIT )+
            cnt3 = 0
            while True: #loop3
                alt3 = 2
                LA3_0 = self.input.LA(1)

                if ((48 <= LA3_0 <= 57)) :
                    alt3 = 1


                if alt3 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:278:11: DIGIT
                    pass
                    self.mDIGIT()


                else:
                    if cnt3 >= 1:
                        break #loop3

                    eee = EarlyExitException(3, self.input)
                    raise eee

                cnt3 += 1





            self._state.type = _type
            self._state.channel = _channel

        finally:

            pass

    # $ANTLR end "INT"



    def mTokens(self):
        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:8: ( WS | COMMENT | PROC | ARROW | COOP | IN_KW | LP | RP | LB | RB | SC | COM | AND | OR | NOT | IDENT | INT )
        alt4 = 17
        alt4 = self.dfa4.predict(self.input)
        if alt4 == 1:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:10: WS
            pass
            self.mWS()


        elif alt4 == 2:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:13: COMMENT
            pass
            self.mCOMMENT()


        elif alt4 == 3:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:21: PROC
            pass
            self.mPROC()


        elif alt4 == 4:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:26: ARROW
            pass
            self.mARROW()


        elif alt4 == 5:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:32: COOP
            pass
            self.mCOOP()


        elif alt4 == 6:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:37: IN_KW
            pass
            self.mIN_KW()


        elif alt4 == 7:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:43: LP
            pass
            self.mLP()


        elif alt4 == 8:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:46: RP
            pass
            self.mRP()


        elif alt4 == 9:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:49: LB
            pass
            self.mLB()


        elif alt4 == 10:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:52: RB
            pass
            self.mRB()


        elif alt4 == 11:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:55: SC
            pass
            self.mSC()


        elif alt4 == 12:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:58: COM
            pass
            self.mCOM()


        elif alt4 == 13:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:62: AND
            pass
            self.mAND()


        elif alt4 == 14:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:66: OR
            pass
            self.mOR()


        elif alt4 == 15:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:69: NOT
            pass
            self.mNOT()


        elif alt4 == 16:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:73: IDENT
            pass
            self.mIDENT()


        elif alt4 == 17:
            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/guard_transitions/translators/pintlang.g:1:79: INT
            pass
            self.mINT()







    # lookup tables for DFA #4

    DFA4_eot = DFA.unpack(
        u"\3\uffff\1\20\1\uffff\2\20\6\uffff\3\20\2\uffff\2\20\1\32\1\20"
        u"\1\34\3\20\1\uffff\1\40\1\uffff\1\41\2\20\2\uffff\4\20\1\50\1\20"
        u"\1\uffff\5\20\1\57\1\uffff"
        )

    DFA4_eof = DFA.unpack(
        u"\60\uffff"
        )

    DFA4_min = DFA.unpack(
        u"\1\11\2\uffff\1\162\1\uffff\1\117\1\156\6\uffff\1\156\1\162\1\157"
        u"\2\uffff\1\157\1\117\1\60\1\144\1\60\1\164\1\143\1\120\1\uffff"
        u"\1\60\1\uffff\1\60\1\145\1\105\2\uffff\1\163\1\122\1\163\1\101"
        u"\1\60\1\124\1\uffff\1\111\1\126\1\111\1\124\1\131\1\60\1\uffff"
        )

    DFA4_max = DFA.unpack(
        u"\1\172\2\uffff\1\162\1\uffff\1\117\1\156\6\uffff\1\156\1\162\1"
        u"\157\2\uffff\1\157\1\117\1\172\1\144\1\172\1\164\1\143\1\120\1"
        u"\uffff\1\172\1\uffff\1\172\1\145\1\105\2\uffff\1\163\1\122\1\163"
        u"\1\101\1\172\1\124\1\uffff\1\111\1\126\1\111\1\124\1\131\1\172"
        u"\1\uffff"
        )

    DFA4_accept = DFA.unpack(
        u"\1\uffff\1\1\1\2\1\uffff\1\4\2\uffff\1\7\1\10\1\11\1\12\1\13\1"
        u"\14\3\uffff\1\20\1\21\10\uffff\1\6\1\uffff\1\16\3\uffff\1\15\1"
        u"\17\6\uffff\1\3\6\uffff\1\5"
        )

    DFA4_special = DFA.unpack(
        u"\60\uffff"
        )


    DFA4_transition = [
        DFA.unpack(u"\2\1\25\uffff\1\1\7\uffff\1\7\1\10\2\uffff\1\14\1\4"
        u"\1\uffff\1\2\12\21\1\uffff\1\13\5\uffff\2\20\1\5\27\20\1\11\1\uffff"
        u"\1\12\1\uffff\1\20\1\uffff\1\15\7\20\1\6\4\20\1\17\1\16\1\3\12"
        u"\20"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\22"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\23"),
        DFA.unpack(u"\1\24"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\25"),
        DFA.unpack(u"\1\26"),
        DFA.unpack(u"\1\27"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\30"),
        DFA.unpack(u"\1\31"),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u"\1\33"),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u"\1\35"),
        DFA.unpack(u"\1\36"),
        DFA.unpack(u"\1\37"),
        DFA.unpack(u""),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u""),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u"\1\42"),
        DFA.unpack(u"\1\43"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\44"),
        DFA.unpack(u"\1\45"),
        DFA.unpack(u"\1\46"),
        DFA.unpack(u"\1\47"),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u"\1\51"),
        DFA.unpack(u""),
        DFA.unpack(u"\1\52"),
        DFA.unpack(u"\1\53"),
        DFA.unpack(u"\1\54"),
        DFA.unpack(u"\1\55"),
        DFA.unpack(u"\1\56"),
        DFA.unpack(u"\12\20\7\uffff\32\20\4\uffff\1\20\1\uffff\32\20"),
        DFA.unpack(u"")
    ]

    # class definition for DFA #4

    DFA4 = DFA




def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import LexerMain
    main = LexerMain(pintlangLexer)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)
